* {
  font-family: Arial, Helvetica, sans-serif;
  scrollbar-width: none;
  -ms-overflow-style: none;
  ::-webkit-scrollbar {
    display: none;
  }
}

body {
  margin: 0 1rem;
  border: 0;
  padding: 0;
}

.code {
  font-family: "Courier New", Courier, monospace;
}

.container {
  margin: 0;
  border: 0;
  padding: 0;
  overflow: scroll;
}

.inline-info {
  word-break: break-all;
}

#main-container {
  margin: 1rem auto;
}

table {
  text-align: center;
  margin-inline: auto;
  th,
  td {
    padding: 0.5rem 1rem;
  }
}